import mimetypes
import os
import smtplib
import time
from email import encoders
from email.header import Header
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr
from os.path import getsize
import logging

logger =logging.getLogger('tkschedule.emails')

# This library is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.  If not, see <http://www.gnu.org/licenses/> or <http://www.gnu.org/licenses/lgpl.txt>.
import string
from tkschedule.ntlm import ntlm
import base64
from smtplib import SMTPException, SMTPAuthenticationError

# def asbase64(msg):
#     return string.replace(base64.encodestring(msg), '\n', '')

def ntlm_authenticate(smtp, username, password):

    code, response = smtp.docmd("AUTH", "NTLM " + ntlm.create_NTLM_NEGOTIATE_MESSAGE(username))
    if code != 334:
        raise SMTPException("Server did not respond as expected to NTLM negotiate message")
    challenge, flags = ntlm.parse_NTLM_CHALLENGE_MESSAGE(response)
    user_parts = username.split("\\", 1)
    code, response = smtp.docmd("", ntlm.create_NTLM_AUTHENTICATE_MESSAGE(challenge, user_parts[1], user_parts[0], password, flags))
    if code != 235:
        raise SMTPAuthenticationError(code, response)

class EmailManager(object):

    _instance = None

    def __new__(cls, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, **kwargs):
        '''
        constructor
        :param kwargs:Variable parameter
        '''
        self.kwargs = kwargs
        self.smtp_server = 'mail01.tkmold.com' #'smtp.163.com'
        self.MAX_FILE_SIZE = 10 * 1024 * 1024

        # initialize the configuration file
        self.__init_cfg()

    def __get_cfg(self, key, throw=True):
        '''
        get the configuration file based on the key
        :param key:
        :param throw:
        :return:
        '''
        cfg = self.kwargs.get(key)
        if throw == True and (cfg is None or cfg == ''):
            raise Exception("The configuration can't be empty", 'utf-8')
        return cfg

    def __init_cfg(self):
        self.msg_from = self.__get_cfg('msg_from')
        self.password = self.__get_cfg('password')
        self.msg_to = ';'.join(self.__get_cfg('msg_to'))
        self.msg_subject = self.__get_cfg('msg_subject')
        self.msg_content = self.__get_cfg('msg_content')
        self.msg_date = self.__get_cfg('msg_date')
        # attachment
        self.attach_file = self.__get_cfg('attach_file', throw=False)

    def login_server(self):
        '''
        login server
        :return:
        '''
        server = smtplib.SMTP_SSL(self.smtp_server, 25)
        server.set_debuglevel(1)
        ntlm_authenticate(server,r'tkmold\trialmold','123456')
        #server.login(self.msg_from, self.password)
        return server

    def get_main_msg(self):
        '''
        suject content
        :return:msg
        '''
        msg = MIMEMultipart()
        # message content
        msg.attach(MIMEText(self.msg_content, 'plain', 'utf-8'))

        msg['From'] = self._format_addr('From <%s>' % self.msg_from)
        msg['To'] = self._format_addr('To <%s>' % self.msg_to)
        msg['Subject'] = Header(self.msg_subject, 'utf-8')
        msg['Date'] = self.msg_date

        # attachment content
        attach_file = self.get_attach_file()
        if attach_file is not None:
            msg.attach(attach_file)
        return msg

    def get_attach_file(self):
        '''
        generate mail attachment content
        :return:
        '''
        if self.attach_file is not None and self.attach_file != '':
            try:
                if getsize(self.attach_file) > self.MAX_FILE_SIZE:
                    raise Exception('The attachment is too large and the upload failed!!')
                with open(self.attach_file, 'rb') as file:
                    ctype, encoding = mimetypes.guess_type(self.attach_file)
                    if ctype is None or encoding is not None:
                        ctype = 'application/octet-stream'
                    maintype, subtype = ctype.split('/', 1)
                    mime = MIMEBase(maintype, subtype)
                    mime.set_payload(file.read())
                    # set header
                    mime.add_header('Content-Disposition', 'attachment',
                                    filename=os.path.basename(self.attach_file))
                    mime.add_header('Content-ID', '<0>')
                    mime.add_header('X-Attachment-Id', '0')
                    # set the attachment encoding rules
                    encoders.encode_base64(mime)
                    return mime
            except Exception as e:
                logger.info('%s......' % e)
                return None
        else:
            return None

    def _format_addr(self, s):
        name, addr = parseaddr(s)
        return formataddr((Header(name, 'utf-8').encode(), addr))

    def set_msg_content(self, content):
        self.msg_content = content

    def set_msg_to(self, tolist):
        self.msg_to = tolist

    def send(self):
        try:
            # initialize the configuration file
            # self.__init_cfg()
            # log on to the SMTP server and verify authorization
            server = self.login_server()
            # mail content
            msg = self.get_main_msg()
            # send mail
            server.sendmail(self.msg_from, self.msg_to, msg.as_string())
            server.quit()
            logger.info("Send succeed!!")
        except smtplib.SMTPException:
            logger.info("Error:Can't send this email!!")


if __name__ == "__main__":
    MAIL_CONf = {
        'msg_from': 'trialmold@tkmold.com',
        'password': '123456',
        # 'msg_from': 'solitary02@163.com',
        # 'password': 'ZGHKSEWBQSMUNGBS',
        # 'msg_to': ['trialmold@tkmold.com'],
        'msg_to': ['1621582331@tkmold.com'],
        'msg_subject': '试模约机确认提醒(请勿回复本邮件)！',
        'msg_content': 'No Content!',
        'attach_file': r'',
        'msg_date': time.ctime()
    }

    manager = EmailManager(**MAIL_CONf)
    manager.set_msg_content("my Content.")
    manager.send()
